
#include <io.h>
#include <locale.h>
#include "GComp.h"
#include "Declarations.h"
#include "CommonFunctions.h"
#include "MMS.h"
#include "Storage.h"
#include "NameTable.h"
#include "Ordering.h"

CGCompParameters TaskParameters;
CNameTable NameTable;


LPCSTR GComp_GetVersion()
{
	return "2.1.1.beta";
}


void GComp_SetDefaults(CGCompParameters* Parameters) 
{
	Parameters->CompoundFileName = NULL;
	Parameters->DirectoryName = NULL;

	Parameters->CompoundType = by_extension;
	Parameters->Verbose = 0;
	Parameters->LogType = LogType_stdout;
	Parameters->NoOrdering = FALSE;
	Parameters->NoProfiles = FALSE;
	Parameters->NoEmptyMxl = FALSE;
	Parameters->TruncateMMS = FALSE;
	Parameters->NotParseForms = FALSE;
	Parameters->NoDefaultsInDialogs = FALSE;
	Parameters->Language = lang_Rus;
	Parameters->SaveVersion = TRUE;
};


void InitializeNameTable()
{
	if( lang_Eng == TaskParameters.Language )
		NameTable.EngConstructor();
	else
		NameTable.RusConstructor();
}

BOOL GComp_Decompile(CGCompParameters* Parameters)
{
	TaskParameters = *Parameters;
	char* locale = setlocale( LC_ALL, ".ACP");
	CMMS* MMS = NULL;
	CString csDirectoryName = TaskParameters.DirectoryName;

	try
	{
		InitializeNameTable();

		CStorage Storage;
		Storage.OpenCompound(CString(TaskParameters.CompoundFileName));
		CStorage TypedTextStorage(Storage, "TypedText");

		if( Storage.GetStorage() == NULL || TypedTextStorage.GetStorage() == NULL )
			return FALSE;
		
		CoInitialize(NULL); //need it for unzip modules

		CNameTableRecord* nt_rec = NameTable.Find(NT_MainMetadataStream);
		if( external_report == TaskParameters.CompoundType )
		{
			MMS = (CMMS*)Storage.ParseStream(nt_rec->StorageName, false);
		}
		else
		{
			Storage.Open(CString("Metadata"));
			MMS = (CMMS*)Storage.ParseStream(nt_rec->StorageName, false);
			Storage.Close();
		}

		if( MMS != NULL )
		{
			CreateDirectory(csDirectoryName); //create directory for decompyle to

			if( TaskParameters.SaveVersion )
			{
				CString Version = GComp_GetVersion();
				CString FName = csDirectoryName + "\\gcomp.ver";
				Storage.CopyStringToFile(Version, FName);
			}

			MMS->Decompile(csDirectoryName, Storage, TypedTextStorage);
			delete MMS;
			Storage.PruneDirectory(csDirectoryName);
		}

		CoUninitialize(); //need it for unzip modules
	}
	/*   -  
	catch(CGCompError* e)
	{
		if( MMS != NULL ) delete MMS;
		e->LogError();
		return false;
	}
	*/
	catch(...)
	{
		if( MMS != NULL ) delete MMS;
		return FALSE;
	}

	if( MMS == NULL )
		return FALSE;
	else
		return TRUE;
}

///////////////////////////////////////////////////////////////////////
//////  Compilation functions  ////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
FILE* AssembleMMS();
extern int MMS_debug;

BOOL GComp_Compile(CGCompParameters* Parameters)
{
	TaskParameters = *Parameters;
	char* locale = setlocale( LC_ALL, ".ACP");
	FILE *mmsFile = NULL;
	CMMSObject* MMSObject = NULL;
	CString csDirectoryName = TaskParameters.DirectoryName;

	try
	{
		InitializeNameTable();

		if( !DirectoryExist(csDirectoryName) )
		{
			Msg(0, "ERR: Source directory '%s' not exist\n", TaskParameters.DirectoryName);
			return FALSE;
		}
		CStorage Storage;
		if( !Storage.CreateCompound(CString(TaskParameters.CompoundFileName)) )
			return FALSE;

		mmsFile = AssembleMMS();
		rewind(mmsFile);
		MMS_InitParser(csDirectoryName+"\\MainMetadataStream");
		if( TaskParameters.Verbose >= 3) MMS_debug = 1;
		int RetCode = MMS_parse(mmsFile, &MMSObject);
		fclose(mmsFile);
		mmsFile = NULL;

		if( MMSObject == NULL )
			return FALSE;

		CoInitialize(NULL);
		CStorage TypedTextStorage(Storage);
		if( TaskParameters.CompoundType != external_report )
			TypedTextStorage.Create(CString("TypedText"));
		MMSObject->Compile(Storage, TypedTextStorage, csDirectoryName);
		CoUninitialize();
	}
	catch(...)
	{
		if( mmsFile != NULL ) fclose(mmsFile);
		if( MMSObject != NULL ) delete MMSObject;
		return false;
	}

	delete MMSObject;
	return TRUE;
}

